﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI.WebControls;
using Dynamicweb.Samples.Web.Model;
using Dynamicweb.Controls;

namespace Dynamicweb.Samples.Web.CustomModules.DvkArticles.Admin
{
	public partial class AddEditArticle : System.Web.UI.Page
	{
		protected void Page_Load(object sender, EventArgs e)
		{
			InitializeStylesGallery();

			if (!Page.IsPostBack && Id > 0)
			{
				BindData();
			}
		}

		/// <summary>
		/// Saves the current article and maintains the active view.
		/// </summary>
		protected void SaveButton_Click(object sender, EventArgs e)
		{
			SaveArticle(false);
		}

		/// <summary>
		/// Saves the current article and redirects the user to the articles list.
		/// </summary>
		protected void SaveAndCloseButton_Click(object sender, EventArgs e)
		{
			SaveArticle(true);
		}

		/// <summary>
		/// Creates a new or updates an existing article in the database.
		/// </summary>
		/// <param name="redirectWhenDone">if set to <c>true</c>, the user is redirected to the main articles page, Default.aspx.</param>
		private void SaveArticle(bool redirectWhenDone)
		{
			Article article;
			using (CustomModulesSamplesEntities db = new CustomModulesSamplesEntities())
			{
				if (Id > 0)
				{
					article = db.Articles.Include("Category").Where(a => a.Id == Id).FirstOrDefault();
					if (updateLastModifed.Checked)
					{
						article.UpdateDateTime = DateTime.Now;
					}
				}
				else
				{
					article = new Article { CreateDateTime = DateTime.Now, UpdateDateTime = DateTime.Now };
					db.AddToArticles(article);
				}

				article.Body = Body.Text;
				int categoryId = Convert.ToInt32(CategoryList.SelectedValue);
				article.Category = db.Categories.Where(c => c.Id == categoryId).FirstOrDefault(); // Will be easier with EF for .NET 4 where you can use foreign keys
				article.CssClass = selectedCssClass.Value;
				article.PublicationDateFrom = PublishFrom.Date;
				article.PublicationDateTo = PublishTo.Date;
				article.SeoDescription = SeoDescription.Text;
				article.Summary = Summary.Text;
				article.Title = ArticleTitle.Text;
				article.Published = published.Checked;

				db.SaveChanges();
				InfoBarSettings.CreateMessage("Article saved successfully", Infobar.MessageType.Information);
				Id = article.Id;

				if (redirectWhenDone)
				{
					EndEditing();
				}
			}
		}

		/// <summary>
		/// Gets an article from the database and prepopulates the form.
		/// </summary>
		private void BindData()
		{
			using (CustomModulesSamplesEntities db = new CustomModulesSamplesEntities())
			{
				Article article = db.Articles.Include("Category").Where(a => a.Id == Id).FirstOrDefault();
				if (article != null)
				{
					ArticleTitle.Text = article.Title;
					Summary.Text = article.Summary;
					SeoDescription.Text = article.SeoDescription;
					Body.Text = article.Body;
					PublishFrom.Date = article.PublicationDateFrom;
					PublishTo.Date = article.PublicationDateTo;
					CategoryList.DataBind();
					ListItem listItem = CategoryList.Items.FindByValue(article.Category.Id.ToString());
					if (listItem != null)
					{
						listItem.Selected = true;
					}
					published.Checked = article.Published;
					selectedCssClass.Value = article.CssClass ?? string.Empty;
				}
				else
				{
					EndEditing();
				}
			}
		}

		/// <summary>
		/// Redirects to the main article page.
		/// </summary>
		private void EndEditing()
		{
			Response.Redirect("Default.aspx");
		}

		#region Style Gallery related functionality


		/// <summary>
		/// Databinds the ListView with a list of CSS classes.
		/// </summary>
		private void InitializeStylesGallery()
		{
			Dictionary<string, string> Styles = new Dictionary<string, string>()
			{
				{"Normal", "Normal"},
				{"HighContrast", "High Contrast"},
				{"Heading1", "Heading 1"},
				{"Heading2", "Heading 2"},
				{"Heading3", "Heading 3"},
				{"Paragraph", "Paragraph"},
				{"Introduction", "Introduction"},
				{"Code", "Code"}
			};

			if (!Page.IsPostBack)
			{
				stylesList.DataSource = Styles;
				stylesList.DataBind();
			}
		}

		/// <summary>
		/// Calculates the RowIndex of the currently created item and updates the RowIndex property.
		/// </summary>
		protected void stylesList_ItemCreated(object sender, ListViewItemEventArgs e)
		{
			ListViewDataItem test = e.Item as ListViewDataItem;
			if (test.DataItemIndex % stylesList.GroupItemCount == 0)
			{
				RowIndex = test.DataItemIndex / stylesList.GroupItemCount;
			}
		}


		/// <summary>
		/// Returns the index for the row being added to the ListView control.
		/// </summary>
		protected internal int RowIndex { get; set; }
		#endregion

		/// <summary>
		/// Tries to get the ID from the Query String or ViewState and return it. Otherwise, it returns 0. 
		/// The assigned value is stored in ViewState so it remaains available when you click Save multiple times.
		/// </summary>
		private int Id
		{
			get
			{
				int id;
				string temp = Request.QueryString["Id"];
				if (!string.IsNullOrEmpty(temp) && int.TryParse(temp, out id))
				{
					return id;
				}
				object o = ViewState["ArticleId"];
				if (o != null)
				{
					return Convert.ToInt32(o);
				}
				return 0;
			}
			set
			{
				ViewState["ArticleId"] = value;
			}
		}
	}
}