﻿using System;
using System.Linq;
using Dynamicweb.Samples.Web.Model;
using Dynamicweb.Controls;
using System.Collections.Generic;
using System.Linq.Dynamic;

namespace Dynamicweb.Samples.Web.CustomModules.DvkArticles.Admin
{
	public partial class Default : System.Web.UI.Page
	{
		protected void Page_Init(object sender, EventArgs e)
		{
			deleteButton.Click += deleteButton_Click;
		}

		protected void Page_Load(object sender, EventArgs e)
		{
			if (!Page.IsPostBack)
			{
				//Prepopulate the Editor with the IntroText if available.
				using (CustomModulesSamplesEntities context = new CustomModulesSamplesEntities())
				{
					SiteSetting mySettings = context.SiteSettings.FirstOrDefault();
					if (mySettings != null)
					{
						IntroText.Text = mySettings.IntroText;
					}
				}
				LoadArticles();
			}
		}

		#region List functionality

		private void LoadArticles()
		{
			using (CustomModulesSamplesEntities context = new CustomModulesSamplesEntities())
			{
				articleList.Clear();
				var allArticles = context.Articles.Include("Category").Where(a => !a.Deleted).OrderBy(SortColumn + " " + SortDirection.ToString());

				foreach (var article in allArticles)
				{
					ListRow row = new ListRow();
					row.AddColumn(article.Title);
					row.AddColumn(article.Category.Description);
					row.OnClientClick = string.Format("editCurrent({0});", article.Id);
					row.ContextMenuID = "articleMenu";
					row.ItemID = "Item" + article.Id.ToString();
					row.RowID = article.Id;
					articleList.AddRow(row);
				}
			}
		}

		protected void articleList_Sort(object sender, ListSortEventArgs e)
		{
			SortDirection = e.Direction;
			SortColumn = e.Column.Name;
			LoadArticles();
		}


		protected void articleList_PageIndexChanged(object sender, EventArgs e)
		{
			LoadArticles();
		}

		public string SortColumn
		{
			get
			{
				object o = ViewState["SortColumn"];
				if (o != null)
				{
					return o as string;
				}
				return "Title";
			}
			set
			{
				if (value.ToLower() == "category")
				{
					value = "Category.Description";
				}
				ViewState["SortColumn"] = value;
			}
		}


		public ListSortDirection SortDirection
		{
			get
			{
				object o = ViewState["ListSortDirection"];
				if (o != null)
				{
					return (ListSortDirection)o;
				}
				return ListSortDirection.Ascending;
			}
			set
			{
				ViewState["ListSortDirection"] = value;
			}
		}

		#endregion


		/// <summary>
		/// Copies the selected articles.
		/// </summary>
		protected void copyButton_Click(object sender, EventArgs e)
		{
			using (CustomModulesSamplesEntities context = new CustomModulesSamplesEntities())
			{
				foreach (int articleId in SelectedArticleIds)
				{
					Article article = context.Articles.Include("Category").Where(a => a.Id == articleId).FirstOrDefault();
					if (article != null)
					{
						Article newArticle = new Article()
						{
							Body = article.Body,
							Category = article.Category,
							CreateDateTime = article.CreateDateTime,
							Deleted = article.Deleted,
							PublicationDateFrom = article.PublicationDateFrom,
							PublicationDateTo = article.PublicationDateTo,
							SeoDescription = article.SeoDescription,
							Summary = article.Summary,
							Title = "Copy of " + article.Title,
							UpdateDateTime = article.UpdateDateTime,
							CssClass = article.CssClass,
							Published = false
						};

						context.AddToArticles(newArticle);
					}
				}
				context.SaveChanges();
				InfoBarSettings.CreateMessage("Articles copied successfully", Infobar.MessageType.Information);
				LoadArticles();
			}
		}

		/// <summary>
		/// Deletes the selected articles.
		/// </summary>
		protected void deleteButton_Click(object sender, EventArgs e)
		{
			using (CustomModulesSamplesEntities context = new CustomModulesSamplesEntities())
			{
				foreach (int articleId in SelectedArticleIds)
				{
					Article article = context.Articles.Where(a => a.Id == articleId).FirstOrDefault();
					if (article != null)
					{
						article.Deleted = true;
					}
				}
				context.SaveChanges();
				InfoBarSettings.CreateMessage("Articles deleted successfully", Infobar.MessageType.Information);
				LoadArticles();
			}
		}

		/// <summary>
		/// Stores the IntroText in the database
		/// </summary>
		protected void saveButton_Click(object sender, EventArgs e)
		{
			using (CustomModulesSamplesEntities context = new CustomModulesSamplesEntities())
			{
				SiteSetting mySettings = context.SiteSettings.FirstOrDefault();
				if (mySettings == null)
				{
					mySettings = new SiteSetting();
					context.AddToSiteSettings(mySettings);
				}
				mySettings.IntroText = IntroText.Text;
				context.SaveChanges();
				InfoBarSettings.CreateMessage("Settings saved successfully", Infobar.MessageType.Information);
				LoadArticles();
			}
		}

		private List<int> SelectedArticleIds
		{
			get
			{
				List<int> tempResult = new List<int>();
				string tempValue = SingleArticleId.Value;
				if (!string.IsNullOrEmpty(tempValue))
				{
					SingleArticleId.Value = string.Empty;
					tempResult.Add(Convert.ToInt32(tempValue));
					return tempResult;
				}

				LoadArticles();
				foreach (ListRow item in articleList.GetSelectedRows())
				{
					tempResult.Add(Convert.ToInt32(item.RowID));
				}

				//foreach (string key in Request.Form.AllKeys.Where(k => k.StartsWith("chk") && k.EndsWith(articleList.ID)))
				//{
				//  int articleId = Convert.ToInt32(key.Split(new char[] { '_' })[1]);
				//  tempResult.Add(articleId);
				//}
				return tempResult;
			}
		}
	}
}